import requests
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials

cred_dict = etld_lib_credentials.get_cred()
url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/2.0/create/am/tag"

tags = {
"qetl-all-ec2": {"ruleText": "aws.ec2.accountId:*", "ruleType": "CLOUD_ASSET",  "provider": "EC2"},
"qetl-all-gcp": {"ruleText": "gcp.compute.instanceId:*", "ruleType": "CLOUD_ASSET",  "provider": "GCP"},
"qetl-all-azure": {"ruleText": "azure.vm.subscriptionId:*", "ruleType": "CLOUD_ASSET",  "provider": "AZURE"},
"qetl-all-hosts": {"ruleText": "return true;", "ruleType": "GROOVY",  "provider": "NONE"}
}

for tag in tags.keys():
    if 'NONE' in tags[tag]['provider']:
        provider_line = ""
    else:
        provider_line = f"<provider>{tags[tag]['provider']}</provider>"

    payload = f"<ServiceRequest><data><Tag><name>{tag}</name><ruleText>{tags[tag]['ruleText']}</ruleText>" \
              f"<ruleType>{tags[tag]['ruleType']}</ruleType>{provider_line}</Tag></data></ServiceRequest>"

    headers = {
      'Content-Type': 'text/xml',
      'X-Requested-With': 'QualysPostman',
      'Accept': 'application/json',
      'Authorization': cred_dict['authorization']
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print(response.text)


#!/usr/bin/env python3
# Manage qetl users.
import argparse
import os
import sys
import re
import logging
import time
import getpass
from pathlib import Path
import qualys_etl.etld_lib.etld_lib_functions as etld_lib_functions
import qualys_etl.etld_lib.etld_lib_config as etld_lib_config
import qualys_etl.etld_lib.etld_lib_credentials as etld_lib_credentials
import qualys_etl.etld_lib.etld_lib_date_time_functions as api_datetime

global qetl_user_home
global found_api_credentials
global command_line_arguments


def validate_username(username):
    username_match = re.fullmatch(r"[-_A-Za-z0-9]+", username)
    if username_match is None:
        return False
    else:
        return True


def validate_api_fqdn_server(api_fqdn_server):
    api_fqdn_server_match = re.fullmatch(r"^.*qualysapi\..*$", api_fqdn_server)  # Matches qualysapi.
    if api_fqdn_server_match is None:
        return False
    else:
        return True


def validate_password(password):
    password_match = re.fullmatch(r"[ ]+", password)
    if password_match is None:
        return True  # found no spaces in password, accept it for now.
    else:
        return False  # found spaces in password


def update_api_fqdn_server(cred):
    print(f"Current api_fqdn_server: {cred.get('api_fqdn_server')}")
    response = input(f"Update api_fqdn_server? ( yes or no ): ")
    if response in 'yes':
        while True:
            new_api_fqdn_server = input(f"Enter new api_fqdn_server: ")
            new_api_fqdn_server = re.sub("https://", '', new_api_fqdn_server)
            new_api_fqdn_server = re.sub("http://", '', new_api_fqdn_server)
            new_api_fqdn_server = re.sub("/", '', new_api_fqdn_server)
            if validate_api_fqdn_server(new_api_fqdn_server):
                break
            else:
                print(f"Not valid qualysapi FQDN, retry.")
        cred['api_fqdn_server'] = new_api_fqdn_server
    pass


def update_username(cred):
    time.sleep(1)
    print(f"\n\nCurrent username: {cred.get('username')} in config: {etld_lib_credentials.cred_file}")
    response = input(f"Update Qualys username? ( yes or no ): ")
    if response in 'yes':
        while True:
            new_username = input(f"Enter new Qualys username: ")
            if validate_username(new_username):
                break
            else:
                print(f"Found invalid characters.  Try again.  Use only AlphaNumeric or underscore in username.")
        cred['username'] = new_username
    pass


def update_password(cred):
    print(f"Update password for username: {cred.get('username')}")
    response = input(f"Update password? ( yes or no ): ")
    if response in 'yes':
        while True:
            new_password = getpass.getpass(f"Enter your Qualys password: ")
            if validate_password(new_password):
                break
            else:
                print(f"Found spaces in password, try again.")
        cred['password'] = new_password
    pass


def if_update_credentials(first_time=False):
    if command_line_arguments.credentials is not None or first_time is not False:
        etld_lib_config.main()
        etld_lib_credentials.main()
        credentials = etld_lib_credentials.get_cred()
        update_username(credentials)
        update_api_fqdn_server(credentials)
        update_password(credentials)
        old_cred = etld_lib_credentials.get_cred()
        if old_cred == credentials:
            print(f"No changes to qualys username, password or api_fqdn_server.")
        else:
            etld_lib_credentials.update_cred(credentials)
            new_credentials = etld_lib_credentials.get_cred()
            etld_lib_functions.logger.info(f"credentials updated.  username: {new_credentials.get('username')} "
                                 f"api_fqdn_server: {new_credentials.get('api_fqdn_server')} ")
            print(f"You have updated your credentials.")
            print(f"  Qualys Username: {new_credentials.get('username')}")
            print(f"  Qualys api_fqdn_server: {new_credentials.get('api_fqdn_server')}\n")


def start_etl_knowledgebase():
    import qualys_etl.etld_knowledgebase.etld_00_spawn_workflow_manager_knowledgebase as etl_kb_spawn_from_qetl_manage_user
    print(f"Starting etl_knowledgebase.  For progress see your {etld_lib_config.kb_log_file}")
    etl_kb_spawn_from_qetl_manage_user.main()
    print(f"End      etl_knowledgebase.  For progress see your {etld_lib_config.kb_log_file}")


def start_etl_host_list():
    import qualys_etl.etld_host_list.etld_00_spawn_workflow_manager_host_list \
        as etl_host_list_spawn_from_qetl_manage_user
    print(f"Starting etl_host_list.  For progress see: {etld_lib_config.host_list_log_file}")
    etl_host_list_spawn_from_qetl_manage_user.main()
    print(f"End      etl_host_list.  For results see:  {etld_lib_config.host_list_log_file}")


def start_etl_host_list_detection():
    import qualys_etl.etld_host_list_detection.etld_00_spawn_workflow_manager_host_list_detection \
       as etl_host_list_detection_spawn_from_qetl_manage_user
    print(f"Starting etl_host_list_detection.  For progress see: {etld_lib_config.host_list_detection_log_file}")
    etl_host_list_detection_spawn_from_qetl_manage_user.main()
    print(f"End      etl_host_list_detection.  For results see:  {etld_lib_config.host_list_detection_log_file}")


def command_line_arguments_datetime():
    if command_line_arguments.datetime is None:
        pass
    else:
        if api_datetime.is_valid_qualys_datetime_format(command_line_arguments.datetime):
            etld_lib_config.qetl_manage_user_selected_datetime = command_line_arguments.datetime
        else:
            print(f"\nInvalid datetime: {str(command_line_arguments.datetime)}, "
                  f"please review format and retry when ready")
            print(f"Option: -d 'YYYY-MM-DDThh:mm:ssZ'")
            exit(1)


def if_execute_etl_module():
    command_line_arguments_datetime()
    if command_line_arguments.execute_etl_module == 'etl_knowledgebase':
        start_etl_knowledgebase()
    elif command_line_arguments.execute_etl_module == 'etl_host_list':
        start_etl_host_list()
    elif command_line_arguments.execute_etl_module == 'etl_host_list_detection':
        ## TODO add fcntl lock on etl_knowledgebase and etl_host_list so they are not run concurrently with etl_host_list_detection.
        ## TODO add fcntl lock check logic to stop running if host list or knowledgebase are already executing.
        start_etl_host_list_detection()
    elif command_line_arguments.execute_etl_module is None:
        pass
    else:
        etld_lib_functions.logger.info(f"Invalid Option: {str(command_line_arguments.execute_etl_module)}, retry when ready.")
        print(f"\nInvalid Option: {str(command_line_arguments.execute_etl_module)}, retry when ready")
        print(f"Options are: -e etl_knowledgebase or -e etl_host_list or -e etl_host_list_detection")
        exit(1)


def if_qualys_test_login(first_time=False):
    if command_line_arguments.test is not None or first_time is not False:
        etld_lib_functions.main(log_level=logging.INFO, my_logger_prog_name='qetl_manage_user')
        etld_lib_config.main()
        etld_lib_credentials.main()
        credentials = etld_lib_credentials.get_cred()
        print(f"Qualys Login Test for {credentials.get('username')} "
              f"at api_fqdn_server: {credentials.get('api_fqdn_server')}\n")
        etld_lib_credentials.test_basic_auth()
        if etld_lib_credentials.login_failed is not True:
            print(f"Testing Qualys Login for {credentials.get('username')} "
                  f"Succeeded at {credentials.get('api_fqdn_server')}\n"
                  f"    with HTTPS Return Code: {etld_lib_credentials.http_return_code}.")
        etld_lib_functions.main(log_level=logging.WARN, my_logger_prog_name='qetl_manage_user')


def if_print_report():
    if command_line_arguments.report is not None:
        print(f"Report on user: {etld_lib_config.qetl_user_home_dir}")
        for path in sorted(etld_lib_config.qetl_user_home_dir.rglob('*')):
            depth = len(path.relative_to(etld_lib_config.qetl_user_home_dir).parts)
            spacer = '    ' * depth
            print(f'{spacer}+ {path.name}')
        print("\n")


def help_message(notes):
    help_mess = f'''
        
    {notes}
        
    usage: qetl_manage_user [-h] -u qetl_USER_HOME_DIR [-e execute etl module] [-c] [-t] [-d] [-r] [-l]
    
    Setup and execute etl module for your qualys qetl users.  
    
    optional arguments:
      -h, --help                show this help message and exit
      -u Home Directory Path, --qetl_user_home_dir Home directory Path
                                   Include prefix opt/qetl/users/[user dir] 
                                   Examples:
                                   1) /home/dgregory/opt/qetl/users/q_username 
                                   2) /opt/qetl/users/q_username
      -e etl module name,     --execute_etl_module module name
                                  Ex.  -e etl_knowledgebase or -e etl_host_list or -e etl_host_list_detection
      -d YYMMDDThh:mm:ssZ,    --datetime        YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date. 
      -c, --credentials       update qualys api user credentials: qualys username, password or api_fqdn_server
      -t, --test              test qualys credentials
      -l, --logs              detailed logs sent to stdout
      -r, --report            brief report of the users directory structure.
     
    '''
    print(f"{help_mess}")


def test_command_line_arguments():
    results = command_line_arguments
    # If no options check for -u.  If -u exists, help_message.  If -u does not exist, continue to new user prompts.
    # If -u is not set, help_message
    if results.qetl_user_home_dir is None:
        help_message(f" "
                     f"Please enter -u [ your opt/qetl/users/ user home directory path ]\n"
                     f"    Note: opt/qetl/users/newuser is the root directory for your qetl userhome directory,\n" 
                     f"          enter a new path including the opt/qetl/users/newuser \n"
                     f"           in the path you have authorization to write to.\n"
                     f"           the prefix to your user directory opt/qetl/users is required.\n"
                     f"    Example:\n"
                     f"             qetl_manage_user -u /opt/qetl/users/[your_user_name]\n"
                     f" ")
        exit(6)  # 6 is for testing the command works. Ex. qetl_manage_user; if [[ "$?" == "6"]]; then : ...

    test_qetl_user_home_dir = Path(results.qetl_user_home_dir).absolute()
    if test_qetl_user_home_dir.parent.parent.is_dir() and \
            os.access(str(test_qetl_user_home_dir.parent.parent), os.W_OK) and \
            test_qetl_user_home_dir.parent.name == 'users' and \
            test_qetl_user_home_dir.parent.parent.name == 'qetl' and \
            test_qetl_user_home_dir.parent.parent.parent.name == 'opt':
        pass
    else:
        help_message(f"Please check permissions on {test_qetl_user_home_dir.parent.parent},\n "
                     f"   You don't appear to have authorization to write to that directory.\n")
        exit(1)

    ##TODO Add datetime methods to test date entered.

    if test_qetl_user_home_dir.is_dir():
        if results.execute_etl_module is None and \
                results.credentials is None and \
                results.test is None and \
                results.logs is None and \
                results.report is None:
            help_message(f"Please select an option for qetl_user: {results.qetl_user_home_dir}")
            exit(1)
    else:
        pass


def get_command_line_arguments(args=None):
    global command_line_arguments
    parser = argparse.ArgumentParser(description='Setup and execute etl for your qualys qetl users.')
    parser.add_argument('-u', '--qetl_user_home_dir', default=None, help="Please enter -u option")
    parser.add_argument('-e', '--execute_etl_module', default=None, help='Execute etl_knowledgebase, etl_host_list or etl_host_list_detection')
    parser.add_argument('-d', '--datetime', default=None,
                        help='YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date')
    parser.add_argument('-c', '--credentials', default=None, action="store_true",
                        help='update qualys api user credentials: qualys uername, password or api_fqdn_server')
    parser.add_argument('-t', '--test', default=None, action="store_true", help='test qualys credentials')
    parser.add_argument('-l', '--logs', default=None, action="store_true", help='detailed logs sent to stdout')
    parser.add_argument('-r', '--report', default=None, action="store_true",
                        help='Brief report of the users directory structure.')
    command_line_arguments = parser.parse_args(args)


def setup_qetl_user_home_dir_environment():
    global qetl_user_home
    global command_line_arguments
    # Reset Logging.
    if command_line_arguments.execute_etl_module is None:
        if command_line_arguments.logs is None:
            etld_lib_functions.main(log_level=logging.WARNING, my_logger_prog_name='qetl_manage_user')
        else:
            etld_lib_functions.main(log_level=logging.INFO, my_logger_prog_name='qetl_manage_user')
    else:
        pass

    # qetl_user_home_dir
    os.environ['qualys_etl_user_home'] = command_line_arguments.qetl_user_home_dir
    etld_lib_config.set_path_qetl_user_home_dir()  # If qetl_user_home_dir is malformed, we abort here.
    if etld_lib_config.qetl_user_home_dir.is_dir():
        # Directory Exists.  Options are test, report, execute, etc...
        pass
    else:
        # Potential New User, Query for confirmation
        time.sleep(1)
        print(f"\nqetl_user_home_dir does not exist: {etld_lib_config.qetl_user_home_dir}")
        response = input(f"Create new qetl_user_home_dir? {etld_lib_config.qetl_user_home_dir} ( yes or no ): ")
        if response == 'yes':
            etld_lib_config.qetl_create_user_dirs_ok_flag = True
            etld_lib_config.main()
            time.sleep(1)
            print(f"\nqetl_user_home_dir created: {etld_lib_config.qetl_user_home_dir}")
            if_update_credentials(first_time=True)
            response = input(f"\nWould you like to test login/logout of Qualys? ( yes or no ): ")
            if response == 'yes':
                print("")
                if_qualys_test_login(first_time=True)
            print(f"\nThank you, exiting.\n")
        else:
            print(f"\nThank you, exiting.\n")
            exit(1)


def main():
    get_command_line_arguments(sys.argv[1:])
    test_command_line_arguments()
    setup_qetl_user_home_dir_environment()
    if_execute_etl_module()
    if_update_credentials()
    if_qualys_test_login()
    if_print_report()


if __name__ == '__main__':
    main()






import fcntl
import multiprocessing
import shutil
import sys
import time
import timeit
from contextlib import redirect_stdout, redirect_stderr

import qualys_etl.etld_lib.etld_lib_config as etld_lib_config
import qualys_etl.etld_lib.etld_lib_credentials as etld_lib_credentials
import qualys_etl.etld_lib.etld_lib_functions as etld_lib_functions

global target_module_to_run
global target_method_in_module_to_run
global spawned_process
global spawned_process_max_run_time
global spawned_process_start_time
global spawned_process_stop_time
global spawned_process_sleep_time
global spawned_process_count_to_status_update
global spawned_process_status_count
global log_file_max_size
global log_file_path
global log_file_rotate_path
global lock_file
global log_dir


def spawn_etl_in_background_with_arg(args_list=[], max_proc=None):
    global spawned_process
    global spawned_process_start_time
    global spawned_process_status_count
    spawn_process_list = []

    # target=etl_host_list_detection
    # name=etl_hld_d_2021_06_03T00:00:00Z_b_000001
    # args=list_of_host_ids
    # Create jobs list  ready to spawn
    for arg in args_list:
        arg: dict
        arg.get('target_module_to_run')
        arg.get('target_method_in_module_to_run')
        arg.get('method_arguments')

        spawned_process = multiprocessing.Process(
            target=arg.get('target_module_to_run'),
            name=arg.get('target_method_in_module_to_run'),
            args=arg.get('method_arguments'))
        spawn_process_list.append(spawned_process)


#    spawned_process = multiprocessing.Process(target=target_module_to_run, name=target_method_in_module_to_run)
#    spawned_process.start()
#    etld_lib_functions.logger.info("Spawned ETL in Background")
#    spawned_process_start_time = timeit.default_timer()
#    spawned_process_status_count = 0
#
#    while spawned_process.is_alive():
#        spawned_process_report_status()
#
#    spawned_process_gracefully_ended()
#    time.sleep(1)


def spawn_etl_in_background():
    global spawned_process
    global spawned_process_start_time
    global spawned_process_status_count

    spawned_process = multiprocessing.Process(target=target_module_to_run, name=target_method_in_module_to_run)
    spawned_process.start()
    etld_lib_functions.logger.info("Spawned ETL in Background")
    spawned_process_start_time = timeit.default_timer()
    spawned_process_status_count = 0

    while spawned_process.is_alive():
        spawned_process_report_status()

    spawned_process_gracefully_ended()
    time.sleep(1)


def spawned_process_report_status():
    now = timeit.default_timer()
    run_time = (now - spawned_process_start_time)
    if run_time > spawned_process_max_run_time:
        terminate_spawned_process()
    else:
        spawned_process_status_update()


def spawned_process_gracefully_ended():
    global spawned_process_stop_time
    spawned_process_stop_time = timeit.default_timer()
    run_time = (spawned_process_stop_time - spawned_process_start_time)
    etld_lib_functions.logger.info(f"Final Runtime: {run_time:,.0f} seconds")


def spawned_process_status_update():
    global spawned_process_status_count

    time.sleep(spawned_process_sleep_time)
    if spawned_process_status_count > spawned_process_count_to_status_update or spawned_process_status_count == 0:
        etld_lib_functions.logger.info(f"Job PID {str(spawned_process.pid)} {target_module_to_run.__name__} "
                             f"job running in background.")
        spawned_process_status_count = 1
    else:
        spawned_process_status_count = spawned_process_status_count + 1


def terminate_spawned_process():
    etld_lib_functions.logger.error(f"Max Run Time: {spawned_process_max_run_time:,.0f} Seconds Exceeded")
    etld_lib_functions.logger.error(f"Please review for issues that are slowing down the program.")
    etld_lib_functions.logger.error(f"Terminating job.")
    spawned_process.terminate()
    spawned_process.join()
    exit(1)


def rotate_log_check():
    if log_file_path.is_file():
        log_file_size = log_file_path.stat().st_size  # In Bytes
        if log_file_size > log_file_max_size:
            shutil.copy2(log_file_path, log_file_rotate_path, follow_symlinks=True)
            fo = open(log_file_path, 'w+')
            fo.close()


def etl_main():
    try:
        with open(lock_file, 'wb+') as lock_program_fcntl:        # If locked, exit.
            fcntl.flock(lock_program_fcntl, fcntl.LOCK_EX | fcntl.LOCK_NB)
            rotate_log_check()
            if log_dir.is_dir():
                with open(log_file_path, 'a', newline='', encoding='utf-8') as log_fo:
                    with redirect_stdout(log_fo), redirect_stderr(sys.stdout):
                        etld_lib_functions.main(my_logger_prog_name=target_module_to_run.__name__)
                        etld_lib_config.main()
                        etld_lib_credentials.main()
                        spawn_etl_in_background()
    except Exception as e:
        print(f"Program is already running.  Please re-run when ready. {__file__} ")
        print(f"Exception: {e}")
        exit(1)




import base64
import time
import requests
from pathlib import Path
import shutil
import yaml
import oschmod
import re
import os
import stat
from urllib.parse import urlencode, quote_plus
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_date_time_functions as api_datetime
import qualys_etl

global cred_dir
global cookie_file
global cred_file
global use_cookie
global login_failed
global http_return_code


def get_qualys_headers(request=None):
    # 'X-Powered-By': 'Qualys:USPOD1:a6df6808-8c45-eb8c-e040-10ac13041e17:9e42af6e-c5a2-4d9e-825c-449440445cc8'
    # 'X-RateLimit-Limit': '2000'
    # 'X-RateLimit-Window-Sec': '3600'
    # 'X-Concurrency-Limit-Limit': '10'
    # 'X-Concurrency-Limit-Running': '0'
    # 'X-RateLimit-ToWait-Sec': '0'
    # 'X-RateLimit-Remaining': '1999'
    # 'Keep-Alive': 'timeout=300, max=250'
    # 'Connection': 'Keep-Alive'
    # 'Transfer-Encoding': 'chunked'
    # 'Content-Type': 'application/xml'
    if request is None:
        pass
    else:
        request_url = request.url
        url_fqdn = re.sub("(https://)([0-9a-zA-Z\.\_\-]+)(/.*$)", "\g<2>", request_url)
        url_end_point = re.sub("(https://[0-9a-zA-Z\.\_\-]+)/", "", request_url)
        x_ratelimit_limit = request.headers['X-RateLimit-Limit']
        x_ratelimit_window_sec = request.headers['X-RateLimit-Window-Sec']
        x_ratelimit_towait_sec = request.headers['X-RateLimit-ToWait-Sec']
        x_ratelimit_remaining = request.headers['X-RateLimit-Remaining']
        x_concurrency_limit_limit = request.headers['X-Concurrency-Limit-Limit']
        x_concurrency_limit_running = request.headers['X-Concurrency-Limit-Running']
        headers = {'url': request_url,
                   'api_fqdn_server': url_fqdn,
                   'api_end_point': url_end_point,
                   'x_ratelimit_limit': x_ratelimit_limit,
                   'x_ratelimit_window_sec': x_ratelimit_window_sec,
                   'x_ratelimit_towait_sec': x_ratelimit_towait_sec,
                   'x_ratelimit_remaining': x_ratelimit_remaining,
                   'x_concurrency_limit_limit': x_concurrency_limit_limit,
                   'x_concurrency_limit_running': x_concurrency_limit_running}
        return headers


def update_cred(new_cred):
    cred_example_file_path = Path(etld_lib_functions.qetl_code_dir, "qualys_etl", "etld_templates", ".etld_cred.yaml")
    destination_file_path = Path(cred_dir, ".etld_cred.yaml")
    # Get Current .etld_cred.yaml file
    with open(cred_file, 'r', encoding='utf-8') as cred_yaml_file:
        current_cred = yaml.safe_load(cred_yaml_file)
    # Get Template
    with open(str(cred_example_file_path), "r", encoding='utf-8') as cred_template_file:
        cred_template_string = cred_template_file.read()
    # Update Template # username: initialuser  password: initialpassword  api_fqdn_server: qualysapi.qualys.com
    if current_cred == new_cred:
        pass
    else:
        new_username = f"username: '{new_cred.get('username')}'"
        new_password = f"password: '{new_cred.get('password')}'"
        new_api_fqdn_server = f"api_fqdn_server: '{new_cred.get('api_fqdn_server')}'"
        local_date = api_datetime.get_local_date()
        cred_template_string = re.sub('\$DATE', local_date, cred_template_string)
        cred_template_string = re.sub('username: initialuser', new_username, cred_template_string)
        cred_template_string = re.sub('password: initialpassword', new_password, cred_template_string)
        cred_template_string = re.sub('api_fqdn_server: qualysapi\.qualys\.com', new_api_fqdn_server, cred_template_string)
        with open(str(cred_file), 'w', encoding='utf-8') as cred_file_to_update:
            cred_file_to_update.write(cred_template_string)


def get_cred():
    """ Return dict - api_fqdn_url, authorization, username, password """
    if not Path.is_file(cred_file):
        cred_example_file_path = Path(etld_lib_functions.qetl_code_dir, "qualys_etl", "etld_templates", ".etld_cred.yaml")
        destination_file_path = Path(cred_dir, ".etld_cred.yaml")
        shutil.copy(str(cred_example_file_path), str(destination_file_path), follow_symlinks=True)
        cred_example_file = open(str(cred_example_file_path), "r", encoding='utf-8')
        cred_example = cred_example_file.read()
        cred_example_file.close()
        local_date = api_datetime.get_local_date() # Add date updated to file
        cred_example = re.sub('\$DATE', local_date, cred_example)
        cred_file_example = open(str(cred_file), 'w', encoding='utf-8')
        cred_file_example.write(cred_example)
        cred_file_example.close()

    oschmod.set_mode(str(cred_file), "u+rw,u-x,go-rwx")
    try:
        with open(cred_file, 'r', encoding='utf-8') as cred_yaml_file:
            cred = yaml.safe_load(cred_yaml_file)
            api_fqdn_server = cred.get('api_fqdn_server')
            authorization = 'Basic ' + \
                            base64.b64encode(f"{cred.get('username')}:{cred.get('password')}".encode('utf-8')).decode('utf-8')
            username, password = base64.b64decode(authorization.replace("Basic ", "")).decode('utf-8').split(":")
            etld_lib_functions.logger.info(f"Found your subscription credentials file:  {cred_file}")
            etld_lib_functions.logger.info(f"username: {username}, api_fqdn_server:  {api_fqdn_server}")
            cred_file_mode = stat.filemode(os.stat(cred_file).st_mode)
            etld_lib_functions.logger.info(f" ** Notice: Ensure Credential File permissions are correct for your company.")
            etld_lib_functions.logger.info(f" ** Notice: Credentials File: {cred_file}")
            etld_lib_functions.logger.info(f" ** Permissions are: {cred_file_mode} for {cred_file}")
            return {'api_fqdn_server': api_fqdn_server,
                    'authorization': authorization,
                    'username': username,
                    'password': password}
    except Exception as e:
        etld_lib_functions.logger.error(f"Please add your subscription credentials to the:  {cred_file}")
        etld_lib_functions.logger.error(f"   ** Warning: Ensure Credential File permissions are correct for your company.")
        etld_lib_functions.logger.error(f"   ** Warning: Credentials File: {cred_file}")
        cred_file_mode = stat.filemode(os.stat(cred_file).st_mode)
        etld_lib_functions.logger.error(f"   ** Permissions are: {cred_file_mode} for {cred_file}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def get_cookie():
    try:
        with open(cookie_file, 'r', encoding='utf-8') as f:
            cookie = f.read().replace('\n', '').replace('\r', '')
    except Exception as e:
        etld_lib_functions.logger.error(f"Error in File: {__file__} Line: {etld_lib_functions.lineno()}")
        etld_lib_functions.logger.error(f"               Credentials Dir:  {cred_dir}")
        etld_lib_functions.logger.error(f"              Credentials File:  {cred_file}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)

    return cookie


def qualys_login():
    global use_cookie
    global login_failed
    global http_return_code
    login_failed = True
    cred_dict = get_cred()
    """Login to Qualys, return cookie.  Keep cookie if """
    url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
    payload = {'action': 'login', 'username': cred_dict['username'], 'password': cred_dict['password']}
    payload = urlencode(payload, quote_via=quote_plus)

    headers = {
        'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
        'Content-Type': 'application/x-www-form-urlencoded',
    }
    headers['User-Agent'] = f"qualysetl_v{qualys_etl.__version__}"

    try:
        response = requests.request("POST", url, headers=headers, data=payload)
        http_return_code = response.status_code
        if response.status_code == 200:
            cookie_dict = response.cookies.get_dict()
            cookie = f"DWRSESSIONID={cookie_dict['DWRSESSIONID']}; QualysSession={cookie_dict['QualysSession']}"
            with open(cookie_file, 'w', encoding='utf-8') as cookiefile:
                cookiefile.write(cookie)
            etld_lib_functions.logger.info(f"LOGIN - Qualys Login Success with user: {cred_dict['username']}")
            login_failed = False
        else:
            etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user: {cred_dict['username']}")
            etld_lib_functions.logger.error(f"       HTTP {response.status_code}")
            etld_lib_functions.logger.error(f"       Verify Qualys username, password and api_fqdn_server in Credentials File")
            etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
            etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
            etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
            exit(1)
        use_cookie = True
    except requests.exceptions.RequestException as e:
        etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user")
        etld_lib_functions.logger.error(f"       Verify Qualys username, password and api_fqdn_server in Credentials File")
        etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
        etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
        etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def qualys_logout():
    global use_cookie
    cred_dict = get_cred()
    url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
    payload = {'action': 'logout'}
    headers = {
        'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
        'Content-Type': 'application/x-www-form-urlencoded',
        'Cookie': get_cookie()
    }
    headers['User-Agent'] = f"qualysetl_v{qualys_etl.__version__}"
    try:
        response = requests.request("POST", url, headers=headers, data=payload)
        if response.status_code == 200:
            etld_lib_functions.logger.info(f"LOGOUT - Qualys Logout Success with user: {cred_dict['username']}")
        else:
            etld_lib_functions.logger.warning(f"LOGOUT FAILED - probably stale cookie, continue with warning")
    except Exception as e:
        etld_lib_functions.logger.warning(f"LOGOUT FAILED, probably connectivity issue, continue with warning")
        etld_lib_functions.logger.warning(f"Exception: {e}")

    use_cookie = False


def main():
    global cred_dir
    global cookie_file
    global cred_file
    global use_cookie
    cred_dir = Path(etld_lib_config.qetl_user_cred_dir)  # Credentials Directory
    cookie_file = Path(cred_dir, ".etld_cookie")  # Cookie File
    cred_file = Path(cred_dir, ".etld_cred.yaml")  # YAML Format Qualys Credentials
    use_cookie = False  # Set to true by qualys_login()
    # Override if running from


def test_basic_auth():
    qualys_login()
    time.sleep(2)
    qualys_logout()
    time.sleep(0.5)


if __name__ == '__main__':
    etld_lib_functions.main()
    etld_lib_config.main()
    main()
    test_basic_auth()



import csv
import shelve
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
global count_host_ids_written


def prepare_csv_cell(csv_cell, csv_column):
    if csv_cell is None:
        csv_cell = ""
    elif 'DATE' in csv_column:  # Prepare dates for use in Excel or Database
        csv_cell = csv_cell.replace("T", " ").replace("Z", "")
    elif 'ASSET_GROUP_IDS' in csv_column:  # Prepare IDS for use in Excel
        csv_cell = csv_cell.replace(",", "\n")
    elif not isinstance(csv_cell, str):  # Flatten Nested XML into String
        flatten = etld_lib_functions.flatten_nest(csv_cell)
        csv_cell = ""
        for key in flatten.keys():
            csv_cell = f"{csv_cell}{key}:{flatten[key]}\n"
    return csv_cell


def host_list_to_csv():  # Create CSV File from Shelve Database
    global count_host_ids_written
    host_list_csv_file = etld_lib_config.host_list_csv_file     # Output CSV File
    csv_columns = etld_lib_functions.host_list_csv_columns()         # Host List Columns
    count_host_ids_written = 0
    csv_headers = {}
    for header in csv_columns:
        csv_headers[header] = ""
    try:
        with open(host_list_csv_file, 'w', newline='', encoding='utf-8') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=csv_columns, quoting=csv.QUOTE_ALL)
            writer.writeheader()
            csv_row = csv_headers.copy()
            # Iterate through Shelve Database
            with shelve.open(str(etld_lib_config.host_list_shelve_file), flag='r') as shelve_database:  # Open Host List Shelve DB
                for shelve_database_item in shelve_database:
                    host_list_item = shelve_database[shelve_database_item]
                    for csv_column in csv_columns:  # Iterate through expected columns (contract)
                        if csv_column in host_list_item.keys():  # Iterate through columns found in Shelve
                            host_list_item[csv_column] = prepare_csv_cell(
                                                         host_list_item[csv_column], csv_column)
                            # TODO add truncated field list to data.  low priority.
                            truncated_field_list = ""
                            csv_row[csv_column], truncated_field_list = \
                                etld_lib_functions.truncate_csv_cell(
                                                  max_length=etld_lib_config.host_list_csv_truncate_cell_limit,
                                                  csv_cell=host_list_item[csv_column],
                                                  truncated_field_list=truncated_field_list,
                                                  csv_column=csv_column)
                        else:
                            csv_row[csv_column] = ""  # Ensure blank is added to each required empty field
                    # Write CSV row, Prepare for next row.
                    writer.writerow(csv_row)
                    count_host_ids_written = count_host_ids_written + 1
                    csv_row = csv_headers.copy()

    except Exception as e:
        etld_lib_functions.logger.error(f"Error in File: {__file__} Line: {etld_lib_functions.lineno()}")
        etld_lib_functions.logger.error(f"Error writing to file: {str(host_list_csv_file)}, please retry after fixing error")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def start_msg_host_list_csv():
    etld_lib_functions.logger.info(f"start")


def end_msg_host_list_csv():
    global count_host_ids_written
    etld_lib_functions.logger.info(f"count_host_ids_written rows written to csv: {count_host_ids_written:,}")
    etld_lib_functions.log_file_info(etld_lib_config.host_list_shelve_file, 'in')
    etld_lib_functions.log_dbm_info(etld_lib_config.host_list_shelve_file)
    etld_lib_functions.log_file_info(etld_lib_config.host_list_csv_file)
    etld_lib_functions.logger.info(f"end")


def host_list_csv():
    host_list_to_csv()


def main():
    start_msg_host_list_csv()
    host_list_csv()
    end_msg_host_list_csv()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='host_list_csv')
    etld_lib_config.main()
    main()



import shelve
import json
import requests
import time
import re
from pathlib import Path
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials
global http_error_codes_v2_api
import qualys_etl

http_error_codes_v2_api = {
    "202": "Retry Later Duplicate Operation.",
    "400": "Bad Request Unrecognized parameter",
    "401": "Unauthorized check credentials",
    "403": "Forbidden User account is inactive or user license not authorized for API. ",
    "409": "Conflict Check Concurrency and Rate Limits",
    "501": "Internal Error. Contact your TAM to open ticket. ",
    "503": "Maintenance we are performing scheduled maintenance on our system.",
}


def get_qualys_headers(request=None):
    # 'X-Powered-By': 'Qualys:USPOD1:a6df6808-8c45-eb8c-e040-10ac13041e17:9e42af6e-c5a2-4d9e-825c-449440445cc8'
    # 'X-RateLimit-Limit': '2000'
    # 'X-RateLimit-Window-Sec': '3600'
    # 'X-Concurrency-Limit-Limit': '10'
    # 'X-Concurrency-Limit-Running': '0'
    # 'X-RateLimit-ToWait-Sec': '0'
    # 'X-RateLimit-Remaining': '1999'
    # 'Keep-Alive': 'timeout=300, max=250'
    # 'Connection': 'Keep-Alive'
    # 'Transfer-Encoding': 'chunked'
    # 'Content-Type': 'application/xml'
    if request is None:
        pass
    else:
        request_url = request.url
        url_fqdn = re.sub("(https://)([0-9a-zA-Z\.\_\-]+)(/.*$)", "\g<2>", request_url)
        url_end_point = re.sub("(https://[0-9a-zA-Z\.\_\-]+)/", "", request_url)
        x_ratelimit_limit = request.headers['X-RateLimit-Limit']
        x_ratelimit_window_sec = request.headers['X-RateLimit-Window-Sec']
        x_ratelimit_towait_sec = request.headers['X-RateLimit-ToWait-Sec']
        x_ratelimit_remaining = request.headers['X-RateLimit-Remaining']
        x_concurrency_limit_limit = request.headers['X-Concurrency-Limit-Limit']
        x_concurrency_limit_running = request.headers['X-Concurrency-Limit-Running']
        headers = {'url': request_url,
                   'api_fqdn_server': url_fqdn,
                   'api_end_point': url_end_point,
                   'x_ratelimit_limit': x_ratelimit_limit,
                   'x_ratelimit_window_sec': x_ratelimit_window_sec,
                   'x_ratelimit_towait_sec': x_ratelimit_towait_sec,
                   'x_ratelimit_remaining': x_ratelimit_remaining,
                   'x_concurrency_limit_limit': x_concurrency_limit_limit,
                   'x_concurrency_limit_running': x_concurrency_limit_running}
        return headers


def get_http_error_code_message_v2_api(http_error=""):
    global http_error_codes_v2_api

    if http_error in http_error_codes_v2_api.keys():
        return http_error_codes_v2_api[http_error]
    else:
        return None


def load_json(load_json_file=None, shelve_db=None):
    try:
        with open(load_json_file, "w", encoding='utf-8') as output_json_file:
            output_json_file.write("[")
            with shelve.open(str(shelve_db), flag='r') as shelve_database:
                count_key_value_pairs_loaded_to_json = 0
                shelve_length = len(shelve_database)
                keys_max_count_added_to_json = 1
                for shelve_key in shelve_database:
                    shelve_data = shelve_database[shelve_key]
                    json.dump(shelve_data, output_json_file, indent=4)
                    keys_max_count_added_to_json = keys_max_count_added_to_json + 1
                    count_key_value_pairs_loaded_to_json = count_key_value_pairs_loaded_to_json + 1
                    if keys_max_count_added_to_json > shelve_length:
                        break
                    else:
                        output_json_file.write(",")
            output_json_file.write("]")
    except Exception as e:
        etld_lib_functions.logger.error(f"Error in File: {__file__} Line: {etld_lib_functions.lineno()}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)
    return count_key_value_pairs_loaded_to_json


def extract_qualys(
        try_extract_max_count=3,
        url=None,
        headers=None,
        payload=None,
        http_conn_timeout=30,
        chunk_size_calc=20480,
        xml_file=None,
        cred_dict=None,
        qualys_headers_dict=None,
        multi_proc_batch_number=None
):

    for _ in range(try_extract_max_count):
        try:
            headers['User-Agent'] = f"qualysetl_v{qualys_etl.__version__}"
            with requests.request("POST", url, stream=True, headers=headers, data=payload, timeout=http_conn_timeout) as r:
                #  TODO: grab header and check concurrent connections information
                qualys_headers = get_qualys_headers(r)
                if multi_proc_batch_number is None:
                    qualys_headers_dict['batch_000001'] = get_qualys_headers(r)
                else:
                    qualys_headers_dict[multi_proc_batch_number] = get_qualys_headers(r)

                etld_lib_functions.logger.info(f"Qualys Headers: {qualys_headers}")
                if r.status_code == 200:
                    with open(xml_file, "w", encoding='utf-8') as f:
                        for chunk in r.iter_content(chunk_size=chunk_size_calc):
                            try:
                                #  TODO: add chunks to queue, use queue limited to last 2, concat and check for errors.
                                f.write(chunk.decode('utf-8'))
                            except Exception as e:
                                f.write(etld_lib_functions.remove_low_high_values(chunk).decode('utf-8'))
                else:
                    message = get_http_error_code_message_v2_api(str(r.status_code))
                    etld_lib_functions.logger.error(f"HTTP USER: {cred_dict['username']} url: {url}")
                    etld_lib_functions.logger.error(f"HTTP {r.status_code}, exiting. message={message}")
                    exit(1)
        except Exception as e:
            etld_lib_functions.logger.warning(f"Warning for extract file: {Path(xml_file).name}")
            etld_lib_functions.logger.warning(f"Warning {e}")
            etld_lib_functions.logger.warning(f"Retry attempt number: {_ + 1}")
            time.sleep(60)
            continue
        else:
            break  # success
    else:
        etld_lib_functions.logger.error(f"Max retries attempted: {try_extract_max_count}")
        etld_lib_functions.logger.error(f"extract file: {Path(xml_file).name}")
        exit(1)



#!/usr/bin/env python3
import sys
import timeit
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials

import qualys_etl.etld_host_list.host_list_process_00_extract as host_list_extract
import qualys_etl.etld_host_list.host_list_process_01_transform_to_shelve as host_list_to_shelve
import qualys_etl.etld_host_list.host_list_process_02_load_json as host_list_json
import qualys_etl.etld_host_list.host_list_process_03_load_csv as host_list_csv
import qualys_etl.etld_host_list.host_list_process_04_load_sqlite as host_list_sqlite
import qualys_etl.etld_host_list.host_list_process_05_distribution as host_list_distribution

global start_time
global stop_time


def host_list_extract_wrapper():
    etld_lib_functions.logger.info(f"start host_list_extract from Qualys")
    etld_lib_functions.logger.info(f"     vm_processed_after={etld_lib_config.host_list_vm_processed_after}")
    host_list_extract.main()
    etld_lib_functions.logger.info(f"end   host_list_extract from Qualys")


def host_list_to_shelve_wrapper():

    etld_lib_functions.logger.info(f"start host_list_shelve xml to shelve")
    host_list_to_shelve.main()
    etld_lib_functions.logger.info(f"end   host_list_shelve xml to shelve")


def host_list_to_json_wrapper():
    etld_lib_functions.logger.info(f"start host_list_json transform Shelve to JSON")
    host_list_json.main()
    etld_lib_functions.logger.info(f"end   host_list_json transform Shelve to JSON")


def host_list_to_csv_wrapper():
    etld_lib_functions.logger.info(f"start host_list_csv - shelve to csv")
    host_list_csv.main()
    etld_lib_functions.logger.info(f"end   host_list_csv - shelve to csv")


def host_list_to_sqlite_wrapper():
    etld_lib_functions.logger.info(f"start host_list_sqlite transform Shelve to Sqlite3 DB")
    host_list_sqlite.main()
    etld_lib_functions.logger.info(f"end   host_list_sqlite transform Shelve to Sqlite3 DB")


def host_list_distribution_wrapper():
    try:
        does_dir_exist = etld_lib_config.host_list_export_dir
        try:
            etld_lib_functions.logger.info(f"start host_list_distribution")
            host_list_distribution.main()
            etld_lib_functions.logger.info(f"end   host_list_distribution")
        except Exception as e:
            etld_lib_functions.logger.info(f"host_list_distribution ended with an {e}, ignore if distribution is off.")
    except Exception as e:
        # no distribution directory found and that's ok.
        pass


def host_list_start_wrapper():
    global start_time
    start_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"__start__ host_list_etl_workflow {str(sys.argv)}")
    etld_lib_functions.logger.info(f"data directory: {etld_lib_config.qetl_user_data_dir}")
    etld_lib_functions.logger.info(f"config file:    {etld_lib_config.qetl_user_config_settings_yaml_file}")
    etld_lib_functions.logger.info(f"cred yaml file: {etld_lib_credentials.cred_file}")
    etld_lib_functions.logger.info(f"cookie file:    {etld_lib_credentials.cookie_file}")


def host_list_end_wrapper():
    global start_time
    global stop_time

    stop_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"runtime for host_list_etl_workflow in seconds: {stop_time - start_time:,}")
    etld_lib_functions.logger.info(f"__end__ host_list_etl_workflow {str(sys.argv)}")


def host_list_etl_workflow():
    try:
        host_list_start_wrapper()
        host_list_extract_wrapper()
        host_list_to_shelve_wrapper()
        host_list_to_json_wrapper()
        host_list_to_csv_wrapper()
        host_list_to_sqlite_wrapper()
        host_list_distribution_wrapper()
        host_list_end_wrapper()
    except Exception as e:
        etld_lib_functions.logger.error(f"Error occurred, please investigate {sys.argv}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def main():
    host_list_etl_workflow()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='host_list_etl_workflow')
    etld_lib_config.main()
    etld_lib_credentials.main()
    main()



#!/usr/bin/env python3
import requests
import time
import re
from pathlib import Path
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials
from qualys_etl.etld_lib.etld_lib_sqlite_tables import get_q_knowledgebase_min_max_dates
from qualys_etl.etld_lib import etld_lib_extract_transform_load_distribute as etld_lib_extract_transform_load_distribute

global url
global payload
global use_cookie
global qualys_headers
global kb_last_modified_after
global xml_file


def setup_vars_required_for_direct_execution_of_main():
    global kb_last_modified_after
    global xml_file
    (min_date, max_date) = get_q_knowledgebase_min_max_dates(etld_lib_config.kb_sqlite_file)
    # TODO add date_time function to convert YYYY-MM-DDThh:mm:ssZ into date - 7 days.
    # TODO take guess work out of knowledgebase updates using last modified - 7 from db.
    etld_lib_functions.logger.info(f"Found Q_KnowledgeBase Min Date: {min_date} Max Date: {max_date}")
    if str(min_date).__contains__("1999"):
        kb_last_modified_after = re.sub(" .*$", "T00:00:00Z", max_date)
        etld_lib_functions.logger.info(f"Found knowledgebase max date of: {max_date}")
        etld_lib_config.kb_last_modified_after = kb_last_modified_after
        etld_lib_functions.logger.info(f"     using kb_last_modified_after={kb_last_modified_after}")
    else:
        etld_lib_functions.logger.info(f"Did not find full knowledgebase, rebuilding...")
        etld_lib_functions.logger.info(f"     using kb_last_modified_after=1970-01-01T00:00:00Z")
        etld_lib_config.kb_last_modified_after = '1970-01-01T00:00:00Z'
        kb_last_modified_after = etld_lib_config.kb_last_modified_after

    try:
        xml_file
    except:
        xml_file = etld_lib_config.kb_xml_file


def check_initial_user(cred_dict):
    if cred_dict['username'] == 'initialuser':
        etld_lib_functions.logger.error(f"Please create your credentials using qetl_manage_user. User is default user {cred_dict['username']}")
        exit(1)


def knowledgebase_extract():
    global url
    global payload
    global use_cookie
    global qualys_headers
    global kb_last_modified_after
    global xml_file

    cred_dict = etld_lib_credentials.get_cred()
    check_initial_user(cred_dict)
    authorization = cred_dict['authorization']
    use_cookie = etld_lib_credentials.use_cookie
    url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/knowledge_base/vuln/"

    payload = {'action': 'list', 'details': 'All', 'show_disabled_flag': '1', 'show_qid_change_log': '1',
               'show_supported_modules_info': '1', 'show_pci_reasons': '1',
               'last_modified_after': kb_last_modified_after}

    etld_lib_functions.logger.info(f"api call    - {url}")
    etld_lib_functions.logger.info(f"api options - {payload}")
    etld_lib_functions.logger.info(f"cookie      - {use_cookie}")

    if use_cookie is False:
        headers = {'X-Requested-With': 'qualysetl', 'Authorization': authorization}
    else:
        headers = {'X-Requested-With': 'qualysetl', 'Cookie': etld_lib_credentials.get_cred()}

    # TODO: create method to build payload.  Allow users options to adjust payload.
    try_extract_max_count = 3
    chunk_size_calc = 20480
    try_extract_max_count = 3
    http_conn_timeout = 30  #
    qualys_headers = {}
    multi_proc_batch_number = None
    etld_lib_extract_transform_load_distribute.extract_qualys(
        try_extract_max_count=try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=http_conn_timeout,
        chunk_size_calc=chunk_size_calc,
        xml_file=xml_file,
        cred_dict=cred_dict,
        qualys_headers_dict=qualys_headers,
        multi_proc_batch_number=multi_proc_batch_number)


def start_msg_knowledgebase_extract():
    etld_lib_functions.logger.info(f"start")


def end_msg_knowledgebase_extract():
    global url
    global qualys_headers
    global kb_last_modified_after
    global xml_file
    etld_lib_functions.log_file_info(url, 'in')
    etld_lib_functions.log_file_info(xml_file)
    for h in qualys_headers.keys():
        etld_lib_functions.logger.info(f"Qualys Header: {h} = {qualys_headers[h]}")
    etld_lib_functions.logger.info(f"end")


def main():
    start_msg_knowledgebase_extract()
    setup_vars_required_for_direct_execution_of_main()
    knowledgebase_extract()
    end_msg_knowledgebase_extract()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='knowledgebase_extract')
    etld_lib_config.main()
    etld_lib_credentials.main()
    main()



from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_sqlite_tables as etld_lib_sqlite_tables
from qualys_etl.etld_knowledgebase import knowledgebase_process_04_load_sqlite
from qualys_etl.etld_host_list import host_list_process_04_load_sqlite
from qualys_etl.etld_lib import etld_lib_sqlite_tables
global host_list_detection_updated_csv_columns


def host_list_detection_sqlite():
    global host_list_detection_updated_csv_columns
    etld_lib_sqlite_tables.create_table_no_primary_key(etld_lib_config.host_list_detection_table_name,
                                           host_list_detection_updated_csv_columns,
                                           etld_lib_config.host_list_detection_sqlite_file)
    etld_lib_sqlite_tables.bulk_insert_csv_file(etld_lib_config.host_list_detection_table_name,
                                    etld_lib_config.host_list_detection_csv_file,
                                    host_list_detection_updated_csv_columns,
                                    etld_lib_config.host_list_detection_sqlite_file)


def start_msg_host_list_detection_sqlite():
    etld_lib_functions.logger.info(f"start")


def end_msg_host_list_detection_sqlite():
    etld_lib_functions.logger.info(f"count host_id rows added to table: {etld_lib_sqlite_tables.count_rows_added_to_table:,}")
    etld_lib_functions.log_file_info(etld_lib_config.host_list_detection_csv_file, 'in')
    etld_lib_functions.log_file_info(etld_lib_config.host_list_detection_sqlite_file)
    etld_lib_functions.logger.info(f"end")


def combine_csv_columns_host_and_detection():
    global host_list_detection_updated_csv_columns
    csv_host_columns = etld_lib_functions.host_list_detection_csv_columns()
    csv_host_detection_columns = etld_lib_functions.host_list_detection_qid_csv_columns()
    csv_columns = csv_host_columns + csv_host_detection_columns
    idx = 0
    for column in csv_host_columns:
        if column in 'DETECTION_LIST':
            csv_columns.__delitem__(idx)
        idx = idx + 1
    host_list_detection_updated_csv_columns = csv_columns


def main():
    start_msg_host_list_detection_sqlite()
    combine_csv_columns_host_and_detection()
    host_list_detection_sqlite()
    knowledgebase_process_04_load_sqlite.kb_sqlite_file = etld_lib_config.host_list_detection_sqlite_file
    knowledgebase_process_04_load_sqlite.main()
    host_list_process_04_load_sqlite.host_list_sqlite_file = etld_lib_config.host_list_detection_sqlite_file
    host_list_process_04_load_sqlite.main()
    etld_lib_sqlite_tables.create_table_q_knowledgebase_in_host_list_detection(etld_lib_config.host_list_detection_sqlite_file)
    end_msg_host_list_detection_sqlite()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='host_list_detection_sqlite')
    etld_lib_config.main()
    main()

